# Copyright (c) 2007 Resolver Systems

# 2007-10-24 mjfoord Version 0.1
# 2007-10-25 mjfoord Version 0.1.1, keyword arguments
# 2007-11-11 sanxiyn Exception handling

import clr
clr.AddReference('Python.Runtime')

from System import Array, IntPtr

from Python.Runtime import (
    PythonEngine,
    PyDict, PyFloat, PyInt, PyIter, PyList, PyLong, 
    PyNumber, PyObject, PySequence, PyString, PyTuple,
    PythonException
)


engine = PythonEngine()
engine.Initialize()


def lock():
    h = engine.AcquireLock()
    def unlock():
        engine.ReleaseLock(h)
    return unlock
    

def GIL(function):
    def f(*args, **keywargs):
        unlock = lock()
        try:
            ret = function(*args, **keywargs)
        finally:
            unlock()
        return ret
    return f


def handle_exception(function):
    def f(*args, **kw):
        try:
            return function(*args, **kw)
        except PythonException, e:
            exc = PyExcConvert(PyObject(e.PyType))
            value = ConvertToIpy(PyObject(e.PyValue))
            raise exc(value)
    return f


@GIL
def Import(name):
    module = engine.ImportModule(name)
    if module is None:
        raise ImportError("Importing module named %s failed" % name)
    return PythonObject(module)


@GIL
def _getBools():
    "Hack because 'FromManagedObject' doesn't work for bools"
    builtin = engine.ImportModule('__builtin__')
    return {False: builtin.GetAttr('False'), True: builtin.GetAttr('True')}

_bools = _getBools()


##################


def PyDictConvert(obj):
    newDict = PyDict()
    for key, value in obj.items():
        newDict[ConvertToPy(key)] = ConvertToPy(value)
    return newDict

def PyTupleConvert(obj):
    members = Array.CreateInstance(PyObject, len(obj))
    for i, item in enumerate(obj):
        members[i] = ConvertToPy(item)
    return PyTuple(members)

def PyListConvert(obj):
    members = Array.CreateInstance(PyObject, len(obj))
    for i, item in enumerate(obj):
        members[i] = ConvertToPy(item)
    return PyList(members)

def PyExcConvert(obj):
    module = str(obj.GetAttr('__module__'))
    name = str(obj.GetAttr('__name__'))
    return getattr(__import__(module), name)


def IpyDictConvert(obj):
    container = PyDict(obj).Items()
    return dict((ConvertToIpy(container[i][0]), ConvertToIpy(container[i][1])) 
                for i in range(container.Length())) 

def IpyTupleConvert(obj):
    pyObj = PyTuple(obj)
    return tuple(ConvertToIpy(pyObj[i]) for i in range(pyObj.Length()))


def IpyListConvert(obj):
    pyObj = PyList(obj)
    return [ConvertToIpy(pyObj[i]) for i in range(pyObj.Length())]


def ConvertToPy(obj):
    if obj is None:
        return PyObject.FromManagedObject(obj)
    elif isinstance(obj, bool):
        return _bools[obj]
    elif type(obj) == PythonObject:
        return obj._real_
    
    pyType = pyTypes.get(type(obj), PyObject)
    return pyType(obj)


def ConvertToIpy(obj):
    if obj.obj == IntPtr.Zero:
        return None
    elif PyTuple.IsTupleType(obj):
        return IpyTupleConvert(obj)
    elif PyDict.IsDictType(obj):
        return IpyDictConvert(obj)
    elif PyList.IsListType(obj):
        return IpyListConvert(obj)
    elif PyFloat.IsFloatType(obj):
        return PyObject.AsManagedObject(obj, float)
    
    try:
        return obj.AsManagedObject(object)
    except TypeError:
        return PythonObject(obj)


def StringDictToPy(obj):
    newDict = PyDict()
    for key, value in obj.items():
        newDict[PyString(key).InvokeMethod('encode')] = ConvertToPy(value)
    return newDict


pyTypes = {
    dict: PyDictConvert,
    float: PyFloat,
    int: PyInt,
    list: PyListConvert,
    long: PyLong,
    tuple: PyTupleConvert,
    str: PyString,
}


##################
    

class PythonObject(object):
    def __init__(self, real):
        self._real_ = real
        
    @GIL
    @handle_exception
    def __getattribute__(self, name):
        real = object.__getattribute__(self, '_real_')
        if name == '_real_':
            return real
        elif name in ('__class__', '__bases__', '__call__'):
            # Would be nice to proxy these - but we only
            # have the chance because of the IronPython bug
            return object.__getattribute__(self, name)
        ##
        return ConvertToIpy(real.GetAttr(name))
        
        
    @GIL
    @handle_exception
    def __setattr__(self, name, value):
        if name == '_real_':
            object.__setattr__(self, '_real_', value)
            return
        ##
        self._real_.SetAttr(name, ConvertToPy(value))


    @GIL
    @handle_exception
    def __getitem__(self, name):
        # This code is never called - which is a bug in IronPython
        # The method has to exist though, or an attribute error is raised!
        return ConvertToIpy(self._real_.GetItem(name))
    
    
    @GIL
    @handle_exception
    def __setitem__(self, name, value):
        # This code is never called - which is a bug in IronPython
        # The method has to exist though, or an attribute error is raised!
        self._real_.SetItem(name, ConvertToPy(value))


    @GIL
    @handle_exception
    def __call__(self, *args, **keywargs):
        if not keywargs:
            return ConvertToIpy(self._real_.Invoke(ConvertToPy(args)))
        return ConvertToIpy(self._real_.Invoke(ConvertToPy(args), StringDictToPy(keywargs)))

